/* global sc, sca */

let objects = require('common/objects');
let beans = require('common/beans');
let PackageInputStream = Packages.agorum.roi.common.utils.Packager.PackageInputStream;
let Charset = java.nio.charset.Charset;


function work(data, parameters) {
  
  let object = objects.tryFind(data.uuid);
  if (object && parameters) {

    let target = objects.find(parameters.target);
    let mailTarget = objects.find(parameters.mailTarget);
    let unzip = (parameters.unzip || '').toLowerCase() === 'true';
    let regex = parameters.regex || '.*';

    let forEachEntry = (source, is, callback) => {
      let pis = new PackageInputStream(is, Charset.forName('Cp850'));

      try {
        let entry;
        while (!!(entry = pis.nextEntry)) {      
          if (callback(source, entry.name, entry.isDirectory() ? null : pis) === false) {
            // stop iteration if callback returns false
            return false;
          }
        }
      }
      finally {
        //pis.closePackage();
      }
    };

    let handle = (source, name, stream, object) => {
      if (!stream && !object) {
        // skip folders
        return;
      }

      name = name.split('/').pop();

      if (unzip && /[\.]zip$/i.test(name)) {
        forEachEntry(source, object ? object.contentStream : stream, handle);
      }
      else if (RegExp(regex, 'i').test(name)) {
        // no regular expression or regular expression matches the name
        let created;

        if (object) {
          // copy attachments directly
          created = objects.copy(object, target);
        }
        else {
          // extract from zip
          created = objects.create('file', {
            name: name,
            target: target,
            content: stream
          });
        }

        // add relation back to the original mail
        if (parameters.attach === 'relation') {
          created.setAttachments([ source ]);
        }
        else if (parameters.attach) {
          beans.set(created, parameters.attach, source.UUID);
        }
      }
    };

    (object.mailAttachments || [])
    .forEach(attachment => handle(object, attachment.name, null, attachment));

    // move mail
    objects.remove(object);
    objects.add(object, mailTarget);

  }
}

module.exports = {
  work: work
};

/**
// Test
let transaction = require('common/transaction');

let parameters = {
  regex: '[\.](pdf)$', //'[\.](pdf|DOC|docx)$', // Beispiel für unterschiedliche Dateiendungen
  // Zielordner in den die Attachements gelegt werden sollen
  target: '/agorum/roi/Files/agorum core open example unwrap attachments/archive/attachments',
  //der Ordner in den die Mails nach Verarbeitung archiviert werden sollen
  mailTarget: '/agorum/roi/Files/agorum core open example unwrap attachments/archive/mails', 
  // Möglichkeiten: 1. Als Metadatum: '~agorum_open_example_unwrap_attachments_orig_mail' oder als Relation: 'relation'  
  attach: '~agorum_open_example_unwrap_attachments_orig_mail',  
  // Hier kann eingeschaltet werden, ob aus zip-Anhängen die Dateien herausgeholt werden sollen, die auf "regex" passen
  unzip: 'true'  
};

//let object = objects.find('1197638');
let object = objects.find('1199176');  // Mail zum testen vorgeben per ID
transaction(() => {
  work({uuid: object.UUID}, parameters);
});


/**/